<?php

/**
 * Created by Evgenii Ioffe
 * @author Evgenii Ioffe <ioffe@umispec.ru>
 * @copyright Copyright (c) 2019, Evgenii Ioffe
 */
class OfdFermaApi
{
    private $debug = false;
    private $logPath = '';
    private $email = '';

    /**
     * @param $login
     * @param $password
     * @return OfdFermaApiAuthTokenModel
     * @throws publicException
     */
    public function getAuthToken($login, $password)
    {
        $result = $this->request('/Authorization/CreateAuthToken', array(
            'Login' => $login,
            'Password' => $password
        ));

        if (!isset($result['AuthToken']) || !isset($result['ExpirationDateUtc'])) {
            $this->logMessage('Неверный формат ответа');
        }

        $token = new OfdFermaApiAuthTokenModel($result['AuthToken'], strtotime($result['ExpirationDateUtc']));

        return $token;
    }

    public function addReceipt(OfdFermaApiReceiptModel $receipt, $authToken)
    {
        $items = array();

        foreach ($receipt->getItems() as $receiptItem) {
            if (!$receiptItem instanceof OfdFermaApiReceiptItemModel) {
                continue;
            }

            $items[] = array(
                'Label' => $receiptItem->getLabel(),
                'Price' => $receiptItem->getPrice(),
                'Quantity' => $receiptItem->getQuantity(),
                'Amount' => $receiptItem->getAmount(),
                'Vat' => $receiptItem->getVat()
            );
        }

        if (!count($items)) {
            throw new publicException('Для заказа не передан список товаров');
        }

        $data = array(
            'Inn' => $receipt->getInn(),
            'Type' => $receipt->getType(),
            'InvoiceId' => $receipt->getInvoiceId() . '-' . $receipt->getType(),
            'LocalDate' => date('Y-m-d\TH:i:s', $receipt->getDate()),
            'CustomerReceipt' => array(
                'TaxationSystem' => $receipt->getTaxationSystem(),
                'Email' => $receipt->getEmail(),
                'Phone' => $receipt->getPhone(),
                'Items' => $items
            ),
        );

        $result = $this->request('/kkt/cloud/receipt?AuthToken=' . $authToken, array(
            'Request' => $data
        ));

        if (!isset($result['ReceiptId'])) {
            $this->logMessage('Неверный формат ответа');
        }

        return $result['ReceiptId'];
    }

    public function getReceiptStatus($receiptId, $authToken)
    {
        $result = $this->request('/kkt/cloud/status?AuthToken=' . $authToken, array(
            'Request' => array(
                'ReceiptId' => $receiptId
            )
        ));

        if(!isset($result['StatusCode'])) {
            $this->logMessage('Неверный формат ответа');
        }

        $status = new OfdFermaApiReceiptStatusModel();
        $status->setCode($result['StatusCode']);
        $status->setName($result['StatusName']);
        $status->setMessage($result['StatusMessage']);
        $status->setModifiedDate(strtotime($result['ModifiedDateUtc']));
        $status->setReceiptDate(strtotime($result['ReceiptDateUtc']));
        $status->setDeviceId($result['Device']['DeviceId']);
        $status->setRnm($result['Device']['RNM']);
        $status->setZn($result['Device']['ZN']);
        $status->setFn($result['Device']['FN']);
        $status->setFdn($result['Device']['FDN']);
        $status->setFpd($result['Device']['FPD']);

        return $status;
    }

    /**
     * @param boolean $debug
     */
    public function setDebug($debug)
    {
        $this->debug = $debug;
    }

    /**
     * @param string $logPath
     */
    public function setLogPath($logPath)
    {
        $this->logPath = $logPath;
    }

    /**
     * @param string $email
     */
    public function setEmail($email)
    {
        $this->email = $email;
    }

    private function request($url, $data, $method = 'POST')
    {
        $url = 'https://ferma.ofd.ru/api/' . ltrim($url, '/');

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 100000);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        if ($data && $method != 'GET') {
            $encoded = json_encode($data);

            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json',
                'Content-Length: ' . strlen($encoded)
            ));
            curl_setopt($ch, CURLOPT_POSTFIELDS, $encoded);
        }

        $result = curl_exec($ch);
        if (!$result) {
            $this->logMessage(curl_error($ch));
        }

        $return = json_decode($result, true);
        if (!is_array($return) || !isset($return['Status'])) {
            $this->logMessage('Неверный формат ответа');
        }

        if ($return['Status'] == 'Failed') {
            $message = $return['Error']['Message'];
            if ($return['Error']['Code'] && $val = $this->getErrorByCode($return['Error']['Code'], $data)) {
                $message = $val;
            }

            $this->logMessage($message);
        }

        return $return['Data'];
    }

    public function logMessage($message)
    {
        if ($this->debug && $this->logPath) {
            file_put_contents($this->logPath, date('d.m.Y H:i:s') . "\t" . $message . PHP_EOL, FILE_APPEND);
        }

        if ($this->email) {
            $mail = new umiMail();
            $mail->addRecipient($this->email);
            $mail->setSubject('OFD Ferma - Ошибка');
            $mail->setContent($message);
            $mail->commit();
            $mail->send();
        }

        throw new publicException($message);
    }

    private function getErrorByCode($code_id, $data)
    {
        $msg = '';

        SWITCH ($code_id) {
            case 1019:
                $msg = "Идентификатор счета '{$data['Request']['InvoiceId']}' уже существует в ОФД";
                break;
        }

        return $msg;
    }

    private function customErrorHandler($errno, $errstr, $errfile, $errline, array $errcontext)
    {
        $this->logMessage($errstr);

        if (0 === error_reporting()) {
            return false;
        }
        throw new publicException($errstr, 0, $errno, $errfile, $errline);
    }
}